//
// scsv.h: libscsv library header file
//
// (C) Copyright 2016 TinyIT
// Written by Pawel Tatera
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; version 3
// of the License.
//


#ifndef SCSV_H
#define SCSV_H 

// Uncomment these two, if have not included them yet...
//#include <list>
//#include <vector>


#ifdef __cplusplus

class scsv {
	public:

	enum state {
		OK,
		W_STR_EMPTY,
		E_OUT_OF_MEMORY=128,
		E_NULL,
		E_STR_EMPTY,
		E_READ_OPEN, E_READ_FAIL,
		E_SAVE_OPEN, E_SAVE_FAIL,
		E_OUT_OF_RANGE
	};

	// Constructors, destructors and operators.
	scsv(char = ',', char = ',', char = 0, char = 0);

	scsv(const scsv &);
	scsv & operator=(const scsv &);

	~scsv();

	// Config
	void init(char = ',', char = ',', char = 0, char = 0);

	inline char set_iseparator(char c){ return p.iSeparator=c; }
	inline char set_oseparator(char c){ return p.oSeparator=c; }
	inline char set_quote(char c){ return p.quoteCh=c; }
	inline char set_unquote(char c){ return p.unquoteCh=c; }
	inline bool empty_lines(bool el=false){ return p.emptyLines=el; }

	void read_cols(unsigned short = 0, unsigned short * = NULL);
	void quote_cols(unsigned short = 0, unsigned short * = NULL);
	void unquote_cols(unsigned short = 0, unsigned short * = NULL);

	// Input
	int read(const char *const, bool = false);
	int read(std::istream &);

	// Output
	int save(const char *const, bool = false);
	int save(std::ostream &);

	// Direct access to CSV data.
	inline std::size_t rows_count() const { return csvContent.size(); }
	int get(std::vector<char *> &, std::size_t = 0);

	// Tells if provided c-string is quoted in context of object created
	// based on this class. The set_unquote needs to be used to set
	// the tested character.
	const char * str_is_quoted(const char *) const;

	private:
	struct {
		char iSeparator;
		char oSeparator;
		char quoteCh;
		char unquoteCh;
		unsigned short needColsL, *needCols;
		unsigned short quoteColsL, *quoteCols;
		unsigned short unquoteColsL, *unquoteCols;
		bool emptyLines;
	} p;
	state s;
	std::list<std::vector<char *> *> csvContent;

	void csv_free();

	// Stores CSV row location into the container.
	// Takes NULL pointer as an empty CSV line.
	void csv_add(std::vector<char *> *);

	// Content copying
	void copy(const scsv &);
	void csv_copy(const std::list<std::vector<char *> *> &);

	// Data processing
	std::vector<char *> * csv_parse(const char * const);
	std::vector<char *> * row_create();

	const std::vector<char *> * csv_seek(std::size_t);

	const char * find_next_separator(const char * const) const;

	static unsigned short map_column_idx(unsigned short *, unsigned short,
	  unsigned short, unsigned short = 0);
};


#endif // __cplusplus

#endif // SCSV_H

